	.include "equ.inc"
	.include "macro.inc"
	.include "vdc.inc"
;------------------------------------------------------------------------
; ZP variables declatarion
;------------------------------------------------------------------------
	.zp
__ptr   .ds 2
_temp	.ds 2

__farptr .macro
    lda    #bank(\1)
    tam    #\3
    lda    #low(\1)
    sta    <\2
    lda    #high(\1)
    ora    #(\3 * 32)
    sta    <\2+1
	.endm

;------------------------------------------------------------------------
; Interrupt vectors
;------------------------------------------------------------------------
	.code
	.bank 0
        .org $E000

        .include "print5x7.asm"

;------------------------------------------------------------------------
; Reset routine
;------------------------------------------------------------------------
irq_reset:
    sei                                ; disable interrupts
    csh                                ; select the 7.16 MHz clock
    cld                                ; clear the decimal flag
    ldx    #$FF                        ; initialize the stack pointer
    txs
    lda    #$FF                        ; map the I/O bank in the first page
    tam    #0
    lda    #$F8                        ; and the RAM bank in the second page
    tam    #1
    stz    $2000                       ; clear all the RAM
    tii    $2000,$2001,$1FFF

    lda    #%11111101
    sta    irq_disable		       ; initialize irq disable register
    stz    irq_status                  ; reset irq status

    jmp    main

;------------------------------------------------------------------------
; Main program
;------------------------------------------------------------------------
 vdcInitTable:
;       reg  low  hi
    .db $07, $00, $00 ; background x-Scroll register
    .db $08, $00, $00 ; background y-scroll register
    .db $09, $10, $00 ; background map size
    .db $0A, $02, $02 ; horizontal period register
    .db $0B, $1F, $04 ; horizontal display register
    .db $0C, $02, $17 ; vertical sync register
    .db $0D, $DF, $00 ; vertical display register
    .db $0E, $0C, $00 ; vertical display position end register

  ; Test :: Start
  string:
    .db "Hello world! Pcengine rocks!"
;     All possible characters
;     .db $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
;     .db $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
;     .db $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
;     .db $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
;     .db $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F
;     .db $50,$51,$52,$53,$54,$55,$56,$57,$58,$59,$5A,$5B,$5C,$5D,$5E,$5F
;     .db $60,$61,$62,$63,$64,$65,$66,$67,$68,$69,$6A,$6B,$6C,$6D,$6E,$6F
;     .db $70,$71,$72,$73,$74,$75,$76,$77,$78,$79,$7A,$7B,$7C,$7D,$7E,$7F
;     .db $80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F
;     .db $90,$91,$92,$93,$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F
;     .db $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7,$A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
;     .db $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
;     .db $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
;     .db $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
;     .db $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7,$E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
;     .db $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF
  ; Test :: End

main:
    lda    #1
    sta    timer_ctrl ; disable timer

    st0    #$05  ; set vdc control register
    st1    #$00  ; disable vdc interupts
    st2    #$00  ; sprite and bg are disabled

    ; initialize vdc
    lda    #low(vdcInitTable)
    sta    <_temp
    lda    #high(vdcInitTable)
    sta    <_temp+1

    cly
.l0:
        lda    [_temp],Y
        sta    $0000
        iny
        lda    [_temp],Y
        sta    $0002
        iny
        lda    [_temp],Y
        sta    $0003
        iny
        cpy    #24
        bne    .l0

    ; set bat
    st0    #$00
    st1    #$00
    st2    #$00

    st0    #$02

    stz    <_temp
    lda    #$01
    sta    <_temp+1

    cly
bat_row:
    clx
bat_col:
        lda    <_temp
        sta    video_data_l
        lda    <_temp+1
        sta    video_data_h

        inc    <_temp
        bne    .l1
        inc    <_temp+1
.l1:

        inx
        cpx    #32
        bne    bat_col
    iny
    cpy    #24
    bne    bat_row

    ; set palette
    lda    #$04
    stz    color_ctrl
    stz    color_ctrl+1

    stz    color_reg_l
    stz    color_reg_h

    stz    color_data_l
    stz    color_data_h

    lda    #$ff
    sta    color_data_l
    lda    #$01
    sta    color_data_h

    ; Test : begin
    ; set string length
    lda    #28
    sta    <_al

    ; set string pointer
    lda    #low(string)
    sta    <_si
    lda    #high(string)
    sta    <_si+1

    ; set vram address where the font will be rendered
    lda    #$00
    sta    <_di
    lda    #$10
    sta    <_di+1

    jsr    print5x7
    ; Test : end

   ; enable bg display
    st0    #$05
    st1    #$80
    st2    #$00

    cli

loop:
    bra    loop

    rti ; just to be sure :p

;------------------------------------------------------------------------
; Timer interrupt
;------------------------------------------------------------------------
irq_timer:
    stz $1403
    rti

;------------------------------------------------------------------------
; NMI interrupt handler
;------------------------------------------------------------------------
irq_nmi:
    rti

;------------------------------------------------------------------------
; IRQ 1 interrupt handler
;------------------------------------------------------------------------
irq_1:
    rti

;------------------------------------------------------------------------
; IRQ 2 interrupt handler
;------------------------------------------------------------------------
irq_2:
    rti

;------------------------------------------------------------------------
; Vector table
;------------------------------------------------------------------------
        .org $FFF6

        .dw irq_2                    ; irq 2
        .dw irq_1                    ; irq 1
        .dw irq_timer                ; timer
        .dw irq_nmi                  ; nmi
        .dw irq_reset                ; reset

;------------------------------------------------------------------------
; Data:
;------------------------------------------------------------------------
	.data
	.bank 1
        .org $6000
	.include "font.inc"
