        .zp
_buffer .ds 14
_block  .ds 1
        .code
;------------------------------------------------------------------------
; 5x7 font rendering
; Arguments :
;     _al string length
;     _si string pointer
;     _di vram address
;
; Output :
;     _block (zp var) contains the number of 8x8 blocs written
;------------------------------------------------------------------------
print5x7:
    ; -----------------------
    ; Usefull macros
    ; -- 0utput current buffer
output5x7    .macro
    ; 7 lines
    clx
print5x7.\@:
    lda    <_buffer+\1,X
    sta    video_data_l
    st2    #$00
    inx
    cpx    #$07
    bne    print5x7.\@
    .endm

   ; -- Get pointer to the corresponding character data
fetch5x7char .macro
    lda    [_si]
    tay
    lda    asciiTo5x7MapLow,y
    sta    <__ptr
    lda    asciiTo5x7MapHigh,y
    sta    <__ptr+1
    incw   <_si
    .endm

    ; -- Jump to the next 8x8 bloc
jumpToNext8x8 .macro
    vreg   #$00
    ; _di += $0010
    clc
    lda    <_di
    adc    #$10
    sta    video_data_l
    sta    <_di

    lda    <_di+1
    adc    #$00
    sta    video_data_h
    sta    <_di+1

    inc    <_block

    vreg   #$02
    .endm

    ; -----------------------
    ; Code starts here!
    ; Reset the number of 8x8 blocs written
    stz    <_block

    ; Disable display
    stz    disp_cr
    vreg   #$05
    st1    #$00
    st2    #$00

    ; Set vram pointer to $1000 and store it in _di
    vreg   #$00
    lda    <_di
    sta    video_data_l
    lda    <_di+1
    sta    video_data_h

    ; Use the vram data register
    vreg   #$02

    ; Map 5x7 font to data bank
    map    font_5x7

print5x7.begin:
    ;------------------------------------------------
    ; char 0
print5x7.0:
    fetch5x7char

    ; Loop over the 7 lines
    cly
print5x7.0a:
    lda   [__ptr],Y
    tax                 ; (a0 b0 c0 d0 e0  0  0  0)
    stx   <_buffer,Y

    iny
    cpy   #$07
    bne   print5x7.0a

    ; Check if we reached the end of the string
    inc    <_bl
    lda    <_bl
    cmp    <_al
    bne    print5x7.1 ; Not yet, let's fetch next char

    ; Output (a0 b0 c0 d0 e0  0  0  0) and quit
    output5x7 0
    jmp    print5x7.end

    ;------------------------------------------------
    ; char 1
print5x7.1:
    fetch5x7char

    ; Loop over the 7 lines
    cly
    clx
print5x7.1.a:
    lda   [__ptr],Y
                         ; (a1 b1 c1 d1 e1  0  0  0)
    asl    A             ; (b1 c1 d1 e1  0  0  0  0) C: a1
    rol    A             ; (c1 d1 e1  0  0  0  0 a1) C: b1
    rol    A             ; (d1 e1  0  0  0  0 a1 b1) C: c1
    pha                  ; save it
    rol    A             ; (e1  0  0  0  0 a1 b1 c1) C: d1
    and    #7            ; ( 0  0  0  0  0 a1 b1 c1) C: d1
    ora    <_buffer,X    ; (a0 b0 c0 d0 e0 a1 b1 c1)
    sta    <_buffer,X
    pla                  ; (d1 e1 0 0 0 0 0 a1 b1)
    and    #192          ; (d1 e1 0 0 0 0 0  0  0)
    sta    <_buffer+7,X

    inx
    iny
    cpy    #$07
    bne    print5x7.1.a

    ; Output (a0 b0 c0 d0 e0 a1 b1 c1)
    output5x7 0
    jumpToNext8x8

    ; Check if we reached the end of the string
    inc    <_bl
    lda    <_bl
    cmp    <_al
    bne    print5x7.2 ; Not yet, let's fetch next char

    ; Output (d1 e1 0 0 0 0 0 0) and quit
    output5x7 7
    jmp    print5x7.end

    ;------------------------------------------------
    ; char 2
print5x7.2:
    fetch5x7char

    ; Loop over the 7 lines
    clx
    cly
print5x7.2a:
    lda   [__ptr],Y
                         ; (a1 b1 c1 d1 e1  0  0  0)
    lsr    A             ; ( 0 a1 b1 c1 d1 e1  0  0)
    lsr    A             ; ( 0  0 a1 b1 c1 d1 e1  0)
    ora   <_buffer+7,X
    sta   <_buffer+7,X

    inx
    iny
    cpy   #$7
    bne   print5x7.2a

    ; Check if we reached the end of the string
    inc    <_bl
    lda    <_bl
    cmp    <_al
    bne    print5x7.3 ; Not yet, let's fetch next char

    ; Output (d1 e1 a2 b2 c2 d2 e2 0) and quit
    output5x7 7
    jmp    print5x7.end

    ;------------------------------------------------
    ; char 3
print5x7.3:
    fetch5x7char

    ; Loop over the 7 lines
    clx
    cly
print5x7.3a:
    lda   [__ptr],Y
                         ; (a3 b3 c3 d3 e3  0  0  0)
    asl    A             ; (b3 c3 d3 e3  0  0  0  0) C: a3
    sta    <_buffer,X
    lda    <_buffer+7,X  ; (d1 e1 a2 b2 c2 d2 e2  0) C: a3
    adc    #0            ; (d1 e1 a2 b2 c2 d2 e2 a3)
    sta    <_buffer+7,X

    inx
    iny
    cpy   #$7
    bne   print5x7.3a

    ; Output (d1 e1 a2 b2 c2 d2 e2 a3)
    output5x7 7
    jumpToNext8x8

    ; Check if we reached the end of the string
    inc    <_bl
    lda    <_bl
    cmp    <_al
    bne    print5x7.4 ; Not yet, let's fetch next char

    ; Output (b3 c3 d3 e3 0 0 0 0)
    output5x7 0
    jmp    print5x7.end

    ;------------------------------------------------
    ; char 4
print5x7.4:
    fetch5x7char

    ; Loop over the 7 lines
    clx
    cly
print5x7.4a:
    lda   [__ptr],Y
                         ; (a4 b4 c4 d4 e4  0  0  0)
    lsr    A             ; ( 0 a4 b4 c4 d4 e4  0  0)
    lsr    A             ; ( 0  0 a4 b4 c4 d4 e4  0)
    lsr    A             ; ( 0  0  0 a4 b4 c4 d4 e4)
    lsr    A             ; ( 0  0  0  0 a4 b4 c4 d4) C: e4
    ora    <_buffer,X    ; (b3 c3 d3 e3 a4 b4 c4 d4) C: e4
    sta    <_buffer,X
    cla                  ; ( 0  0  0  0  0  0  0  0) C: e4
    rol    A             ; (e4  0  0  0  0  0  0  0)
    sta    <_buffer+7,X

    inx
    iny
    cpy   #$7
    bne   print5x7.4a

   ; Output (b3 c3 d3 e3 a4 b4 c4 d4)
    output5x7 0
    jumpToNext8x8

    ; Check if we reached the end of the string
    inc    <_bl
    lda    <_bl
    cmp    <_al
    bne    print5x7.5 ; Not yet, let's fetch next char

    ; Output (e4 0 0 0 0 0 0 0) and quit
    output5x7 7
    jmp    print5x7.end

    ;------------------------------------------------
    ; char 5
print5x7.5:
    fetch5x7char

    ; Loop over the 7 lines
    clx
    cly
print5x7.5a:
    lda   [__ptr],Y
                         ; (a5 b5 c5 d5 e5  0  0  0)
    lsr   A              ; ( 0 a5 b5 c5 d5 e5  0  0)
    ora   <_buffer+7,X   ; (e4 a5 b5 c5 d5 e5  0  0)
    sta   <_buffer+7,X

    inx
    iny
    cpy   #$7
    bne   print5x7.5a

    ; Check if we reached the end of the string
    inc    <_bl
    lda    <_bl
    cmp    <_al
    bne    print5x7.6 ; Not yet, let's fetch next char

    ; Output (e4 a5 b5 c5 d5 e5 0 0) and quit
    output5x7 7
    jmp    print5x7.end

    ;------------------------------------------------
    ; char 6
print5x7.6:
    fetch5x7char

    ; Loop over the 7 lines
    clx
    cly
print5x7.6a:
    lda   [__ptr],Y
                        ; (a6 b6 c6 d6 e6  0  0  0)
    asl    A            ; (b6 c6 d6 e6  0  0  0  0) C: a6
    rol    A            ; (c6 d6 e6  0  0  0  0 a6) C: b6
    pha                 ; save it
    rol    A            ; (d6 e6  0  0  0  0 a6 b6) C: c6
    and    #3           ; ( 0  0  0  0  0  0 a6 b6) C: c6
    ora    <_buffer+7,X ; (e4 a5 b5 c5 d5 e5 a6 b6) C: c6
    sta    <_buffer+7,X

    pla                 ; (c6 d6 e6  0  0  0  0 a6)
    and    #224         ; (c6 d6 e6  0  0  0  0  0)
    sta    <_buffer,X

    inx
    iny
    cpy   #$7
    bne   print5x7.6a

    ; Output (e4 a5 b5 c5 d5 e5 a6 b6)
    output5x7 7
    jumpToNext8x8

    ; Check if we reached the end of the string
    inc    <_bl
    lda    <_bl
    cmp    <_al
    bne    print5x7.7 ; Not yet, let's fetch next char

    ; Output (c6 d6 e6  0  0  0  0  0) and quit
    output5x7 0
    jmp    print5x7.end

    ;------------------------------------------------
    ; char 7
print5x7.7:
    fetch5x7char

    ; Loop over the 7 lines
    clx
    cly
print5x7.7a:
    lda   [__ptr],Y
                        ; (a7 b7 c7 d7 e7  0  0  0)
    lsr    A            ; ( 0 a7 b7 c7 d7 e7  0  0)
    lsr    A            ; ( 0  0 a7 b7 c7 d7 e7  0)
    lsr    A            ; ( 0  0  0 a7 b7 c7 d7 e7)
    ora    <_buffer,X   ; (c6 d6 e6 a7 b7 c7 d7 e7)
    sta    <_buffer,X

    inx
    iny
    cpy   #$7
    bne   print5x7.7a

    ; Output (c6 d6 e6 a7 b7 c7 d7 e7)
    output5x7 0
    jumpToNext8x8

    ; Check if we reached the end of the string
    inc    <_bl
    lda    <_bl
    cmp    <_al
    beq    print5x7.end ; Quit if we reached it

    jmp print5x7.begin ; Let's start over

print5x7.end:
    rts
